"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const web3_providers_http_1 = __importDefault(require("web3-providers-http"));
const types_1 = require("./types");
class Web3RequestManager {
    constructor(providerOptions) {
        this.providerProtocol = types_1.ProviderProtocol.UNKNOWN;
        switch (Web3RequestManager.detectProviderProtocol(providerOptions.providerUrl)) {
            case types_1.ProviderProtocol.HTTP:
                this.provider = new web3_providers_http_1.default(providerOptions);
                this.providerProtocol = types_1.ProviderProtocol.HTTP;
                break;
            case types_1.ProviderProtocol.WS:
                this.providerProtocol = types_1.ProviderProtocol.WS;
                // TODO
                throw Error('Provider protocol not implemented');
            case types_1.ProviderProtocol.IPC:
                this.providerProtocol = types_1.ProviderProtocol.IPC;
                // TODO
                throw Error('Provider protocol not implemented');
            default:
                // TODO figure out if possible to support generic provider
                throw Error('Provider protocol not supported');
        }
    }
    static detectProviderProtocol(providerUrl) {
        try {
            if (/^http(s)?:\/\//i.test(providerUrl)) {
                return types_1.ProviderProtocol.HTTP;
            }
            else if (/^ws(s)?:\/\//i.test(providerUrl)) {
                return types_1.ProviderProtocol.WS;
            }
            else if (/^ipc:\/\//i.test(providerUrl)) {
                return types_1.ProviderProtocol.WS;
            }
            return types_1.ProviderProtocol.UNKNOWN;
        }
        catch (error) {
            throw Error(`Error detecting provider protocol: ${error.message}`);
        }
    }
    static _defaultRpcOptions(rpcOptions) {
        return {
            id: rpcOptions.id ||
                Math.floor(Math.random() * Number.MAX_SAFE_INTEGER),
            jsonrpc: rpcOptions.jsonrpc || this._DEFAULT_JSON_RPC_VERSION,
            method: rpcOptions.method,
            params: rpcOptions.params,
        };
    }
    async send(rpcOptions, providerCallOptions) {
        try {
            if (this.provider === undefined)
                throw Error('No provider initialized');
            return this.provider.send(Web3RequestManager._defaultRpcOptions(rpcOptions), providerCallOptions);
        }
        catch (error) {
            throw Error(`Error sending: ${error.message}`);
        }
    }
    async subscribe(rpcOptions, providerCallOptions) {
        try {
            if (this.provider === undefined)
                throw Error('No provider initialized');
            return this.provider.subscribe(Web3RequestManager._defaultRpcOptions(rpcOptions), providerCallOptions);
        }
        catch (error) {
            throw Error(`Error subscribing: ${error.message}`);
        }
    }
}
exports.default = Web3RequestManager;
Web3RequestManager._DEFAULT_JSON_RPC_VERSION = '2.0';
